import isEqual from 'fast-deep-equal/react';
import React, { useRef, useEffect } from 'react';

function useDeepCompareCallback(fn, deps) {
    const signal = React.useRef(0);
    const prevDeps = React.useRef(deps);
    if (!isEqual(prevDeps.current, deps)) {
        signal.current += 1;
    }
    prevDeps.current = deps;
    return React.useCallback(fn, [signal.current]);
}

function useDeepCompareEffect(fn, deps) {
    const signal = React.useRef(0);
    const prevDeps = React.useRef(deps);
    if (!isEqual(prevDeps.current, deps)) {
        signal.current += 1;
    }
    prevDeps.current = deps;
    React.useEffect(fn, [signal.current]);
}

function useDeepCompareMemo(fn, deps) {
    const signal = React.useRef(0);
    const prevDeps = React.useRef(deps);
    if (!isEqual(prevDeps.current, deps)) {
        signal.current += 1;
    }
    prevDeps.current = deps;
    return React.useMemo(fn, [signal.current]);
}

function useEventCallback(handler) {
    const handlerRef = React.useRef(handler);
    React.useLayoutEffect(() => {
        handlerRef.current = handler;
    });
    return React.useCallback((...args) => {
        const handle = handlerRef.current;
        return handle(...args);
    }, []);
}

function useInterval(callback, duration) {
    const callbackRef = useRef(callback);
    useEffect(() => {
        callbackRef.current = callback;
    }, [callback]);
    useEffect(() => {
        const tick = () => {
            if (callbackRef.current === undefined)
                return;
            callbackRef.current();
        };
        const id = setInterval(tick, duration);
        return () => clearInterval(id);
    }, [duration]);
}

function usePreviousState(value) {
    const ref = useRef(value);
    useEffect(() => {
        ref.current = value;
    }, [value]);
    return ref.current;
}

function useReactiveRef(value) {
    const ref = React.useRef(value);
    ref.current = value;
    return ref;
}

function useSyncState(initialState) {
    const [state, setState] = React.useState(initialState);
    React.useEffect(() => setState(initialState), [initialState]);
    return [state, setState];
}

export { useDeepCompareCallback, useDeepCompareEffect, useDeepCompareMemo, useEventCallback, useInterval, usePreviousState, useReactiveRef, useSyncState };
